@@ -1,3 +1,33 @@
+# coding=utf-8
+#
+# catkin_lint
+# Copyright (c) 2013-2020 Fraunhofer FKIE
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+#  * Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+#  * Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+#  * Neither the name of the Fraunhofer organization nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
+# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 import unittest
 import catkin_lint.cmake as cmake
 
@@ -64,6 +94,10 @@ class CMakeParserTest(unittest.TestCase):
             self.parse_all('cmd("string that spans\nmultiple lines")'),
             [("cmd", ['string that spans\nmultiple lines'])]
         )
+        self.assertEqual(
+            self.parse_all('cmd("string with \\\nline continuation")'),
+            [("cmd", ['string with line continuation'])]
+        )
         self.assertEqual(
             self.parse_all('cmd("\\\\"\\")'),
             [("cmd", ['\\', '"'])]
@@ -214,6 +248,10 @@ class CMakeParserTest(unittest.TestCase):
             self.parse_all("cmd(one;two three)"),
             [("cmd", ["one", "two", "three"])]
         )
+        self.assertEqual(
+            self.parse_all("cmd(one\\;two three)"),
+            [("cmd", ["one;two", "three"])]
+        )
         self.assertEqual(
             self.parse_all('cmd("one;two" three)'),
             [("cmd", ["one;two", "three"])]
@@ -226,6 +264,26 @@ class CMakeParserTest(unittest.TestCase):
             self.parse_all('cmd(one;"two;three")'),
             [("cmd", ["one", "two;three"])]
         )
+        self.assertEqual(
+            self.parse_all('cmd(one [[two]] three)'),
+            [("cmd", ["one", "two", "three"])]
+        )
+        self.assertEqual(
+            self.parse_all('cmd(one [=[two]]]=] three)'),
+            [("cmd", ["one", "two]]", "three"])]
+        )
+        self.assertEqual(
+            self.parse_all('cmd(one [=[two]==]]=] three)'),
+            [("cmd", ["one", "two]==]", "three"])]
+        )
+        self.assertEqual(
+            self.parse_all('cmd(one [[\ntwo]] three)'),
+            [("cmd", ["one", "two", "three"])]
+        )
+        self.assertEqual(
+            self.parse_all('cmd(one [=[ two [==[ ]=] [=[three ]==] ]=])'),
+            [("cmd", ["one", " two [==[ ", "three ]==] "])]
+        )
         self.assertEqual(
             self.parse_all('if(NOT (A OR B)) endif()'),
             [("if", ["NOT", "(", "A", "OR", "B", ")"]), ("endif", [])]
@@ -290,6 +348,10 @@ class CMakeParserTest(unittest.TestCase):
             self.parse_all('cmd(\\${args})', var={"args": "fail"}),
             [("cmd", ["${args}"])]
         )
+        self.assertEqual(
+            self.parse_all('cmd([[${args}]])', var={"args": "fail"}),
+            [("cmd", ["${args}"])]
+        )
         self.assertEqual(
             self.parse_all('cmd(${args})', var={"args": "\\\\"}),
             [("cmd", ["\\\\"])]
@@ -453,6 +515,16 @@ class CMakeParserTest(unittest.TestCase):
             """),
             [("cmd", ["one", "two", "three"])]
         )
+        self.assertEqual(
+            self.parse_all("""\
+            # initial comment
+            cmd(one #[[ bracket comment ]=]
+            fail # embedded comment ]]
+            two
+            )
+            """),
+            [("cmd", ["one", "two"])]
+        )
 
     def test_line_numbering(self):
         """Test CMake parser line numbering"""
